#ifndef _VisibilityInfo_H__
#define _VisibilityInfo_H__

#include <vector>

#include "VisibilityMesh.h"
#include "HierarchyInterface.h"

namespace GtpVisibility {
  
/** Class storing the visibility information of a scene node. 
*/
class NodeInfo 
{
public:
    NodeInfo(HierarchyNode *node, const int projectedPixels, const int visiblePixels): 
	mNode(node)//, mProjectedPixels(projectedPixels), mVisiblePixels(visiblePixels) 
	{}

	int GetVisiblePixels() const {return mVisiblePixels;}
	int GetProjectedPixels() const {return mProjectedPixels;}

	void SetVisiblePixels(int vis) {mVisiblePixels = vis;}
	void SetProjectedPixels(int vis) {mProjectedPixels = vis;}

	void SetNode(HierarchyNode *node) {mNode = node;}
	HierarchyNode *GetNode() const {return mNode;}

	float ComputeRelativeVisibility() { float result = mVisiblePixels; if (mProjectedPixels) result /= (float)mProjectedPixels; return result;}
	//bool operator<(const NodeInfo& rhs) const {return mNode < rhs.mNode;}

protected:

    /** pointer to the scene node */
    HierarchyNode *mNode;
    
    int mVisiblePixels;
	int mProjectedPixels;
};
  
/** Class storing the visibility information of a mesh.
*/
class MeshInfo 
{
public:
	MeshInfo(Mesh *mesh,  const int projectedPixels, const int visiblePixels): 
	mMesh(mesh), mProjectedPixels(projectedPixels), mVisiblePixels(visiblePixels) 
	{}

	int GetVisiblePixels() const {return mVisiblePixels;}
	int GetProjectedPixels() const {return mProjectedPixels;}

	void SetVisiblePixels(float vis) { mVisiblePixels = vis;}
	void SetProjectedPixels(float vis) { mProjectedPixels = vis;}

	Mesh *GetMesh() const {return mMesh;}
	void SetMesh(Mesh *mesh) {mMesh = mesh;}

	bool operator<(const MeshInfo& rhs) const {return mMesh < rhs.mMesh;}
	//bool operator>(const MeshInfo& rhs) const {return mMesh > rhs.mMesh;}
	//bool operator==(const MeshInfo& rhs) const {return mMesh == rhs.mMesh;}
	
protected:

    /** Pointer to the mesh.
	*/
    Mesh *mMesh;
  
    int mVisiblePixels;
	int mProjectedPixels;
};
  
//TODO: this define shall be replaced by template typedef
#define InfoContainer std::vector
//#define InfoContainer std::set
};


#endif
